# copyright 2017 Yi Wang
# don't distribute without author's explicit authorization

# MindBody API

"""This is where most universal code for the Python examples of API implementation exist.
   Much of the abstracted logic such as filling out credential objects and arrays
   resides in this class."""

SOURCE_NAME = "CBA"
SOURCE_PASSWORD = "4+E1R0zbZQe+R6RbSSa9PQB6ZDs="

USER_NAME = "Siteowner"
USER_PASSWORD = "apitest1234"

SITE_IDS = [-99]

def BuildWsdlUrl(serviceName):
    """This function lets us grab the WSDL for any service we need."""

    return "https://api.mindbodyonline.com/0_5/" + serviceName + "Service.asmx?wsdl"


def CreateStaffCredentials(service, user, password, siteIds):
    """This is a pretty basic method for generating a commonly used field (StaffCredentials)
       in API calls."""
    if(user == None or password == None or siteIds == None):
        return None
    else:
        creds = service.factory.create("StaffCredentials")
        creds.Username = user
        creds.Password = password
        creds.SiteIDs = FillArrayType(service, siteIds, "Int")

        return creds


def FillArrayType(service, elems, arrayType, elemName=None):
    """This method will create an 'ArrayOfX' object and then fill it with the array passed in.
       This takes advantage of the fact that every ArrayOfX object follows the same pattern
       (i.e., ArrayOfLong uses a "long" attribute. ArrayOfInt uses an "int" attribute, etc) a
       vast majority of the time. In the case where this doesn't apply (ArrayOfVisit has a
       capitalized Visit field instead of visit), elemName can be manually set to the desired
       value."""
    if elemName == None:
        elemName = arrayType.lower()

    if elems != None:
        ArrayObject = service.factory.create("ArrayOf" + arrayType)
        setattr(ArrayObject, elemName, elems)
        return ArrayObject
    else:
        return None


def FillAbstractObject(service, objectType, valueDict):
    """This method is useful for creating the "cartItems" and "payments" fields of
       the CheckoutShoppingCart method. It takes in the array currently being built,
       the name in the WSDL of the element to be added (e.g. DebitAccountInfo for payments
       or Package for the Item field of a CartItem."""
    newObject = service.factory.create(objectType)

    for currField in valueDict.keys():
        setattr(newObject, currField, valueDict[currField])

    return newObject


def SetEnumerable(service, enum, value):
    """This method will generate and return an instance of enum.value."""
    if enum == None or value == None:
        return None
    else:
        return getattr(service.factory.create(enum), value)


def FillDefaultCredentials(service, request, siteIDs):
    sourceCreds = service.factory.create('SourceCredentials')
    sourceCreds.SourceName = SOURCE_NAME
    sourceCreds.Password = SOURCE_PASSWORD
    sourceCreds.SiteIDs.int = siteIDs

    userCreds = service.factory.create('UserCredentials')
    userCreds.Username = USER_NAME
    userCreds.Password = USER_PASSWORD
    userCreds.SiteIDs.int = siteIDs

    request.SourceCredentials = sourceCreds
    request.UserCredentials = userCreds
    request.XMLDetail = 'Basic' #"Full", "Basic", "Bare"
    request.PageSize = 2000 #25
    request.CurrentPageIndex = 0

    return request

"""Set siteIDs here to whichever site(s) you wish to make calls upon.
   This example represents site -99, the API Sandbox Site."""


def CreateBasicRequest(service, requestName, siteIDs=SITE_IDS):
    """Returns a request object defined by requestName in the WSDL with
       source and user credentials set."""
    request = service.factory.create(requestName)

    if hasattr(request, 'Request'):
        request = request.Request
    return FillDefaultCredentials(service, request, siteIDs)
 


# -*- coding: utf-8 -*-

# copyright 2017 Yi Wang
# don't distribute without author's explicit authorization

import collections
import sys
import threading
import time
import itertools as it

import query

ACTIVE      = 'ACTIVE'
RESERVED    = 'RESERVED'
CLOSED      = 'CLOSED'
OPEN        = 'OPEN'
OTHER       = 'OTHER'

WAIT_LIMIT  = 4
NAME_LENTGH = 16

DEBUG = 1

queued_players = []

def free_player(player):
    lock = threading.Lock()
    lock.acquire()
    try:
        queued_players.remove(player)
    except ValueError:
        pass
    lock.release()

def free_players(players):
    lock = threading.Lock()
    lock.acquire()
    try:
        for x in players :
            queued_players.remove(x)
    except ValueError:
        pass
    lock.release()

def conv_time(seconds):
    hour = int(seconds / 3600)
    minute = int(seconds / 60)
    second = seconds % 60
    return str(hour) + ':' + str(minute) + ':' + str(second)
    
class count_down_timer(object):
    def __init__(self, interval, court, time_display, call_back):
        self.interval = interval
        self.start_time = 0
        self.started = False
        self.time_display = time_display
        self.court = court
        self.timer = None
        self.printer = None
        self.filler = None
        self.call_back = call_back
    
    def get_timer_display(self):
        return self.time_display
    
    def get_interval(self):
        return self.interval
        
    def set_interval(self, interval):
        self.interval = interval
        
    def round_end(self):
        self.court.round_end()
        self.time_display['fg'] = 'green'
        self.call_back(self.court.get_id())
        self.reset()
        if not self.court.is_empty() :
            self.start()
        
    def start(self):
        if self.started is True:
            return
        self.started = True
        self.start_time = time.time()
        self.printer = threading.Timer(1, self.report_time)
        self.printer.daemon = True
        self.printer.start()
        self.filler = threading.Timer(int(self.interval * 0.60), self.set_court_full)
        self.filler.daemon = True
        self.filler.start()
        self.timer = threading.Timer(self.interval, self.round_end)
        self.timer.daemon = True
        self.timer.start()
        self.court.start()
            
    def reset(self):
        if self.timer:
            self.timer.cancel()
        if self.printer:
            self.printer.cancel()
        if self.filler:
            self.filler.cancel()
        self.start_time = 0
        self.started = False
        self.time_display['fg'] = 'green'
        #self.time_display['text'] = conv_time(self.interval)
    
    def report_time(self):
        remaining = self.interval - int(time.time() - self.start_time)
        if remaining < 60 :
            self.time_display['fg'] = 'red'
        else : 
            self.time_display['fg'] = 'green'
        self.time_display['text'] = conv_time(remaining)
        if remaining > 0 :
            self.printer = threading.Timer(1, self.report_time)
            self.printer.daemon = True
            self.printer.start()
    
    def set_court_full(self):
        self.court.set_full()

class court(object):
    def __init__(self, name, id):
        self.name = name
        self.id = id
        self.state = ACTIVE
        self.cur_players = []
        self.wait_players = []
        self.started = False
        self.full = False

    def is_empty(self):
        if self.cur_players:
            return False
        return True
    
    def get_id(self):
        return self.id
    
    def get_waiting_players(self):
        return self.wait_players
    
    def is_full(self):
        if self.state != ACTIVE:
            return True
        if len(self.wait_players) is WAIT_LIMIT :
            return True
        return False
    
    def set_full(self):
        if self.started: 
            self.full = True
        
    def is_playing(self):
        if len(self.cur_players) > 0:
            return True
        return False
    
    def add(self, player):
        if self.state != ACTIVE:
            return False
        if len(self.cur_players) >= 4 or self.full:
            if len(self.wait_players) >= WAIT_LIMIT :
                return False
            else :
                self.wait_players.append(player)
        else :
            self.cur_players.append(player)
        return True
    
    def remove(self, player):
        if player in self.wait_players:
            self.wait_players.remove(player)
            free_player(player)
        elif player in self.cur_players:
            self.cur_players.remove(player)
            free_player(player)
        
    def is_player_waiting(self, player):
        if player in self.wait_players:
            return True
        return False
    
    def is_player_on_court(self, player):
        if player in self.wait_players:
            return True
        if player in self.cur_players:
            return True
        return False
   
    def start(self, force = False):
        if self.state != ACTIVE :
            return
        if self.started is True and force is False :
            return
        self.started = True
    
    def round_end(self):
        self.started = False
        self.full = False
        players = self.cur_players
        if players :
            free_players(players)
            self.cur_players.clear()
            self.cur_players.extend(self.wait_players)
            self.wait_players.clear()
          
    def reset(self):
        players = self.cur_players
        players.extend(self.wait_players)
        self.cur_players.clear()
        self.wait_players.clear()
        self.full = False
        self.set_state(ACTIVE)      
        if players:
            free_players(players)
    
    def set_state(self, state):
        self.state = state
    
    def get_state(self):
        return self.state       
        
    def __str__(self):
        if self.state is ACTIVE :
            text = ''
            if self.cur_players and self.wait_players:
                text += '{} {}\n\n'.format('Current'.ljust(NAME_LENTGH, ' '), 'Wait:')
                for i, j in it.zip_longest(self.cur_players, self.wait_players, fillvalue = ''):
                    text += '{} {}\n'.format(i.ljust(NAME_LENTGH, ' '), j)
            elif self.cur_players:
                text = 'Current:\n\n' + '\n'.join(self.cur_players)
            else :
                text = 'OPEN'
            return text
        else :
            return("{}".format(self.get_state()))
    


class gym(object):
    def __init__(self):
        self.name = 'CBA'
        self.courts = []
        self.num_courts = 18
        self.players = dict()
        
        for i in range(self.num_courts):
            name = 'court ' + str(i + 1)
            self.courts.append(court(name, i))
        
        self.read_players()
    
    def set_num_courts(self, num_courts):
        self.num_courts = num_courts
    
    def read_players(self):
        if DEBUG == 1:
            self.read_local_db()
        else :
            self.players = query.read_clients()
            self.write_local_db()
        # adding admin
        self.add_player('CBA_ADMIN', 100000000, '1111')
            
    def check_player(self, name, pin):
        player = self.players[name]
        if not player:
            return False
        if player.get_pin() != pin :
            return False
        return True
        
    def get_num_courts(self):
        return self.num_courts
    
    def get_court(self, id):
        return self.courts[id]
    
    def get_players(self):
        return self.players.keys()
    
    def get_waiting_players(self):
        players = list()
        for i in range(self.num_courts):
            court = self.courts[i]
            players.extend(court.get_waiting_players())
        return players
            
    def get_free_players(self):
        names = self.get_players()
        return [x for x in names if x not in queued_players]
    
    def add_player(self, name, pid, pin):
        self.players[name] = query.player(name, pid, pin)

    def remove_player_from_court(self, player):
        for i in range(self.num_courts):
            court = self.courts[i]
            if court.is_player_on_court(player) :
                court.remove(player)
                return i
        return -1
            
    def remove_player_from_wait(self, player):
        for i in range(self.num_courts):
            court = self.courts[i]
            if court.is_player_waiting(player) :
                court.remove(player)
                return i
        return -1
    
    def add_to_court(self, player, court_id):
        if player in queued_players :
            return False
        court = self.courts[court_id]
        if court.add(player):
            queued_players.append(player)
            return True
        return False
        
    def reserve_court(self, court_id):
        self.courts[court_id].set_state(RESERVED)
    
    def reset_court(self, court_id):
        self.courts[court_id].set_state(ACTIVE)
    
    def close_court(self, court_id):
        self.courts[court_id].set_state(CLOSED)
        
    def write_local_db(self):
        with open('CBA_player_info', 'w', encoding = 'utf8') as f: 
            for v in self.players.values() :
                f.write('{}\n'.format(v))
    
    def read_local_db(self):
        with open('CBA_player_info', 'r', encoding = 'utf8') as f: 
            for line in f:
                p = line.strip('\n').split(' ')
                self.add_player(p[0], p[1], p[2])
            

#g = gym()
#g.start()


# -*- coding: utf-8 -*-

# copyright 2017 Yi Wang
# don't distribute without author's explicit authorization

import tkinter as tk
import tkinter.ttk as ttk
import tkinter.messagebox as mb
import tkinter.simpledialog as dlg
import tkinter.scrolledtext as st

import traceback as tb
import sys
import queue
from smart_combobox import smart_combobox
import queue


DEBUG = 1
COURTS_PER_ROW = 6

instruction = """\
To Join a court:
-----------------
1. Choose your login name from the "players" pull down list. 
    -    The login name is your first name concatenated with your last name. 
    -    The first name maybe truncated; 
    -    If there are same names, the last 2 digits of your ID maybe concatenated. 
    -    You can type in first letters for fast lookup.
2. A login dialog will pop up. Input your pin (the last 4 digits of your ID) and click OK.
3. A confirmation dialog will pop up if login successfully; otherwise, an error dialog will show. 
    -    You can try up to 3 times. Contact front desk if you forgot your pin.
4. Choose a court by clicking the court button. 
    -    A court can have up to 4 players playing, and up to 4 players waiting. 
    -    You will be placed on the court if less than 4 players are playing; 
    -    If less than 1/3 of time has left for the round, you will be added to the waiting queue; 
    -    If 4 players are already playing, you will be added to the waiting queue.
5. You can join one and only one court.
6. Your time will become red when it is less than 1 minute.
7. Please give up the court when your time is up.

To withdraw from waiting list:
------------------------------
1. Once joined a court, you can withdraw only if you are in the waiting queue. 
    -    Select your login name from the "waiting players" pull down list and login.    
""".strip()

banner_messages = """\
    -    Training available for juniors of all ages. 
    -    Winter session is now open for registration, act soon!    
    -    Keep your belonging close to you. Keep your unused equipment in your bag.  
    -    Be courteous to your fellow players, watch for people passing by.    
    -    You can ADVERTISE HERE for a fraction of cost! Contact the front desk for details.    
    -    Please share your invaluable feedback. Talk to our staff and/or leave your comment at the front desk!

    REMEMBER, YOU CAN ALWAYS RELY ON THE FRIENDLY CBA STAFF FOR ANY ASSISTANCE!    
"""

play_font           = 'Monaco 10'
state_font          = 'Arial 16 bold'
message_font        = 'Arial 10 italic'
timer_font          = 'Monaco 16 bold'

open_color          = 'pale green'
play_color          = 'light yellow'
full_color          = 'pale violet red'
timer_normal_color  = 'green'

class two_entry_dialog(dlg.Dialog):
    def __init__(self, master, title):
        dlg.Dialog.__init__(self, master, title)
        
    def body(self, master = None):
        tk.Label(master, text = 'courts').grid(row = 0)
        tk.Label(master, text = 'period').grid(row = 1)

        self.e1 = tk.Entry(master)
        self.e2 = tk.Entry(master)

        self.e1.grid(row = 0, column = 1, padx = 10)
        self.e2.grid(row = 1, column = 1, padx = 10)
        return self.e1 # initial focus

    def apply(self):
        first = self.e1.get()
        second = self.e2.get()
        self.result = first, second 
        
class court_widget(object):
    def __init__(self, court, timer, button):
        self.court = court
        self.button = button
        self.timer = timer
        self.timer_display = timer.get_timer_display()       
        
    def start(self):
        self.timer.start()
        
    def reset_to_open(self):
        self.court.reset()
        self.timer.reset()
        self.button['bg'] = open_color
        self.button['font'] = play_font
        self.court['text'] = 'OPEN'
        self.timer_display['fg'] = timer_normal_color  
        
    def reset(self):
        self.court.reset()
        self.timer.reset()  
    
    def set_button_text(self, additional_msg = ''):
        if self.court.get_state() != 'ACTIVE' or self.court.is_empty():
            self.button['font'] = state_font
        else :
            self.button['font'] = play_font
            
        msg = str(self.court)
        if additional_msg != '':
            msg += '\n' + additional_msg
        self.button['text'] = msg
        
    def set_button_bg(self):
        if self.court.is_full() :
            self.button['bg'] = full_color
        elif self.court.is_playing() :
            self.button['bg'] = play_color
        else :
            self.button['bg'] = open_color
    
    def set_interval(self, interval):
        self.timer.set_interval(interval)
    
    def refresh(self):
        self.set_button_text()
        self.set_button_bg()
              
    def update(self, state = 'normal', timer_text = '', additional_msg = ''):
        self.set_button_text(additional_msg)
        self.set_button_bg()
        self.button['state'] = state
        if timer_text != '':
            self.timer_display['text'] = timer_text
         
    def reserve(self, additional_msg):   
        self.reset()
        self.court.set_state('RESERVED')
        self.update('disabled', '0:0:0', additional_msg)
    
    def close(self, additional_msg):   
        self.reset()
        self.court.set_state('CLOSED')
        self.update('disabled', '0:0:0', additional_msg)
        
    def open(self):
        self.court.set_state('ACTIVE')
        self.update('normal', queue.conv_time(self.timer.get_interval()))
 
###############
# Main app
#############                           
class Application(tk.Frame):
    def __init__(self, master):
        tk.Frame.__init__(self, master)
        self.grid()
        self.timer_displays = []
        self.courts = []
        self.cur_player = ''
        if DEBUG == 1 :
            self.interval = 20 # in seconds
        else :
            self.interval = 1200 # 20 minutes
        self.gym = queue.gym()
        self.num_courts = self.gym.get_num_courts()
        self.hide_instruction = True
        self.is_admin = False
        
        # fonts and colors
        self.create_widgets()
        self.toggle_instruction()
        self.hide_admin()

        self.init_all_courts()
                    
    def create_widgets(self):
        # 2 panels inside. 1 for courts, 1 for player
        
        self.pw = tk.PanedWindow(self, orient = "vertical", 
                                 borderwidth = 1, relief = 'solid', sashwidth = 1)
        self.pw.pack(side = tk.TOP, fill = tk.BOTH, expand = 1)
        
        self.title_pane = tk.Frame(self.pw)
        self.court_pane = tk.Frame(self.pw)
        self.top_pane = tk.PanedWindow(self.pw, orient = 'horizontal')
        self.admin_pane = tk.Frame(self.pw) # admin pane adds on demand
        self.pw.add(self.title_pane, pady = 10, padx = 20)
        self.pw.add(self.top_pane, pady = 10, padx = 10)
        self.pw.add(self.court_pane, pady = 0, padx = 10)
        self.player_pane = tk.Frame(self.top_pane, padx = 10)
        self.message_pane = tk.Frame(self.top_pane, padx = 40)
        self.top_pane.add(self.player_pane)
        self.top_pane.add(self.message_pane)
        
        # title pane
        self.title_label = tk.Label(self.title_pane, text = "CBA player queuing system",
                                    font = 'Monaco 16 bold', fg = 'dark green')
        self.title_label.grid(row = 0, column = 0, sticky= tk.N)
        # court pane
        self.court_buttons = []
        for i in range(self.num_courts) :
            r = int(i / COURTS_PER_ROW) * 3
            c = i % COURTS_PER_ROW
            label = tk.Label(self.court_pane, text = "Court {0}".format(i + 1))
            label.grid(row = r, column = c)
            
            button_width_in_pixel = 252
            button_height_in_pixel = 145
            b_frame = tk.Frame(self.court_pane, width = button_width_in_pixel, height = button_height_in_pixel)
            b_frame.rowconfigure(0, minsize = button_height_in_pixel, pad = 0)
            b_frame.columnconfigure(0, minsize = button_width_in_pixel, pad = 0)
            b_frame.grid(row = r + 1, column = c)
            parent = b_frame
            button = tk.Button(parent, text = 'OPEN', bg = open_color,
                               command = lambda court_id = i : self.sel_court(court_id), 
                               #width = 38, height = 14, 
                               justify = tk.LEFT, font = state_font)
            button.grid(row = 0, column = 0, sticky = 'EWNS')
            self.court_buttons.append(button)
            b_frame.grid_propagate(False)

            times = tk.Label(self.court_pane, fg = timer_normal_color, 
                             text = queue.conv_time(self.interval),
                             font = timer_font)
            times.grid(row = r + 2, column = c)
            self.timer_displays.append(times)
             
        # player pane
        tk.Label(self.player_pane, text = "waiting players:").grid(row = 1, column = 0, 
                                                                   padx = 10, sticky= tk.N)
        self.waiting_player_list = smart_combobox(self.player_pane, height = 40)
        self.set_waiting_player_list()
        self.waiting_player_list.set('')
        self.waiting_player_list.bind("<<ComboboxSelected>>", self.waiting_player_selected)
        self.waiting_player_list.bind("<Return>", self.waiting_player_selected)
        self.waiting_player_list.grid(row = 1, column = 1, sticky = tk.N, padx = 10)
        
        tk.Label(self.player_pane, text = "players:").grid(row = 0, column = 0, padx = 10)
        self.player_list = smart_combobox(self.player_pane, height = 40)
        self.set_player_list()
        self.player_list.set('')
        self.player_list.bind("<<ComboboxSelected>>", self.player_selected)
        self.player_list.bind("<Return>", self.player_selected)
        self.player_list.grid(row = 0, column = 1, sticky = tk.W, padx = 10)
        
        self.refresh_player_button = tk.Button(self.player_pane, text = 'Refresh', 
                                               command = self.refresh_players)
        self.refresh_player_button.grid(row = 0, column = 2)
        
        self.inst_button = tk.Button(self.player_pane, text = 'Show Instruction', 
                                     command = self.toggle_instruction)
        self.inst_button.grid(row = 0, column = 3)       
        self.instruction_msg = st.ScrolledText(self.message_pane, font = message_font, 
                                               fg = 'blue', bd = 2, relief = 'ridge',
                                               width = 140, height = 8)
        self.instruction_msg.insert(tk.END, instruction)
        self.instruction_msg.grid(row = 0, column = 0)
        

        # admin pane
        self.admin_close_court_button = tk.Button(self.admin_pane, text = 'Close Courts',
                                                  command = self.admin_close_courts)
        self.admin_reserve_court_button = tk.Button(self.admin_pane, text = 'Reserve Courts',
                                                    command = self.admin_reserve_courts)
        self.admin_set_interval_button = tk.Button(self.admin_pane, text = 'Set Interval',
                                                   command = self.admin_set_interval)
        self.admin_open_button = tk.Button(self.admin_pane, text = 'Open All',
                                           command = self.admin_open_all)
        self.admin_remove_player_button = tk.Button(self.admin_pane, text = 'Remove Player',
                                                    command = self.admin_remove_player)
        self.admin_open_court_button = tk.Button(self.admin_pane, text = 'Open Courts',
                                                    command = self.admin_open_courts)
        
        self.admin_done_button = tk.Button(self.admin_pane, text = 'Done', command = self.admin_done)
        self.admin_exit_button = tk.Button(self.admin_pane, text = 'Exit App', command = exit)
        
        self.admin_close_court_button.grid(row = 0, column = 0, padx = 10, sticky = tk.W)
        self.admin_reserve_court_button.grid(row = 0, column = 1, padx = 10, sticky = tk.W)
        self.admin_set_interval_button.grid(row = 0, column = 2,  padx = 10, sticky = tk.W)
        self.admin_open_button.grid(row = 0, column = 3, padx = 10, sticky = tk.W)
        self.admin_remove_player_button.grid(row = 0, column = 4, padx = 10, sticky = tk.W)
        self.admin_open_court_button.grid(row = 0, column = 5, padx = 10, sticky = tk.W)
        self.admin_done_button.grid(row = 0, column = 10, padx = 10, sticky = tk.W)
        self.admin_exit_button.grid(row = 0, column = 11, padx = 10, sticky = tk.W)
    
    def toggle_instruction(self):
        self.instruction_msg['state'] = 'normal'
        if self.hide_instruction is False :
            self.instruction_msg.delete(1.0, tk.END)
            self.instruction_msg.insert(tk.END, instruction)
            self.instruction_msg['fg'] = 'blue'
            self.inst_button['text'] = 'Hide Instruction'
            self.hide_instruction = True
        else :
            self.instruction_msg.delete(1.0, tk.END)
            self.instruction_msg.insert(tk.END, banner_messages)
            self.instruction_msg['fg'] = 'purple'
            self.inst_button['text'] = 'Show Instruction'
            self.hide_instruction = False
        self.instruction_msg['state'] = 'disabled'        
             
    def waiting_player_selected(self, event):
        player = self.waiting_player_list.get()
        if DEBUG != 1 and not self.login(player) :
            return
        if mb.askyesno('withdraw', 'Withdraw from waiting list?') :
            self.remove_waiting_player(player)
            
    def set_waiting_player_list(self):
        waiting_players = self.gym.get_waiting_players()
        if not waiting_players:
            self.waiting_player_list.set('')
            self.waiting_player_list.set_values('')
        else :
            self.waiting_player_list.set_values(tuple(waiting_players))
            self.waiting_player_list.set('')
        
    def player_selected(self, event):
        player = self.player_list.get()
        if player == 'CBA_ADMIN':
            if self.login(player) :
                self.show_admin()
                self.player_list['state'] = 'disabled'
            return    
        if DEBUG:
            return
        if not self.login(player) :
            return
        mb.showinfo('login', 'login successful, please select court to join!')
        self.cur_player = player
   
    def login(self, player):
        i = 3
        while i >= 0:
            pin = dlg.askstring("pin", "enter your pin", show = "*")
            if not pin:
                self.reset_cur_player()
                self.waiting_player_list.set('')
                return False
            i = i - 1
            if not self.gym.check_player(player, pin):
                if i > 0 :
                    if not mb.askretrycancel('wrong pin', 'wrong pin, retry?'):
                        self.reset_cur_player()
                        self.waiting_player_list.set('')
                        return False
                else :
                    mb.showerror("wrong pin", "3 attempts failed. Please talk to front desk.")
                    self.reset_cur_player()
                    self.waiting_player_list.set('')
                    return False
            else :
                return True
        return False     
       
    def reset_cur_player(self):
        self.cur_player = ''
        self.player_list.set('')
        
    def refresh_players(self):
        self.gym.read_players()
        self.set_player_list()
    
    def remove_waiting_player(self, player):
        if player is None or player == '':
            return
        num = self.gym.remove_player_from_wait(player)
        if num != -1 :
            self.courts[num].refresh()
            self.set_player_list()
            mb.showinfo('Remove Waiting', 'Removed player {} from court {}'.format(player, num + 1))
        else :
            mb.showerror('Remove Waiting', 'Cannot find player {} or player is playing'.format(player))

    def admin_done(self):
        self.player_list['state'] = 'normal'
        self.set_player_list()
        self.hide_admin()
              
    def admin_open_all(self):
        for i in range(self.num_courts):
            self.open_court(i)
            
    def admin_reserve_courts(self):
        d = two_entry_dialog(self, 'Reserve courts')     
        if d.result is None:
            return
        courts = d.result[0].split(' ')
        period = d.result[1].upper()
        for c in courts:
            try:
                n = int(c) - 1
                if n >= 0 and n < self.num_courts:
                    self.reserve_court(n, period)
            except:
                pass
    
    def admin_open_courts(self):
        open_courts = dlg.askstring('Open Courts', 'courts to open')
        if not open_courts:
            return
        for r in open_courts:
            try:
                n = int(r) - 1
                if n >= 0 and n < self.num_courts:
                    self.open_court(n)
            except:
                pass
            
    def admin_close_courts(self):
        d = two_entry_dialog(self, 'Close courts')     
        if d.result is None:
            return
        courts = d.result[0].split(' ')
        period = d.result[1].upper()

        for c in courts:
            try:
                n = int(c) - 1
                if n >= 0 and n < self.num_courts:
                    self.close_court(n, period)
            except:
                pass
            
    def admin_set_interval(self):
        interval = dlg.askinteger("Set Interval", "interval in seconds [0, 3600]")
        if not interval:
            return
        if interval < 0 or interval > 3600 :
            mb.showerror("interval should be in [0, 3600]")
            return
        self.interval = interval
        for c in self.courts:
            c.set_interval(interval)
        for i in range(self.num_courts):
            self.courts[i].set_interval(interval)
            if self.court_buttons[i]['state'] == 'normal':
                if self.gym.get_court(i).is_empty():
                    self.timer_displays[i]['text'] = queue.conv_time(self.interval)

    def admin_remove_player(self):
        player = dlg.askstring("Remove player", "player to remove")
        if player is None:
            return
        num = self.gym.remove_player_from_court(player)
        if num != -1 :
            self.courts[num].refresh()
            self.set_player_list()
                                
    def show_admin(self):
        self.is_admin = True
        self.pw.add(self.admin_pane, pady = 10)
        
    def hide_admin(self):
        self.is_admin = False
        self.pw.remove(self.admin_pane)      
    
    def init_all_courts(self):
        for i in range(self.num_courts):
            court = self.gym.get_court(i)
            timer = queue.count_down_timer(self.interval, court, 
                                           self.timer_displays[i], self.timer_callback)
            self.courts.append(court_widget(court, timer, self.court_buttons[i]))
    
    def start_court(self, num):
        self.courts[num].start()
        
    def timer_callback(self, num):
        self.courts[num].update(timer_text = queue.conv_time(self.interval))
        self.set_player_list()
                
    def set_player_list(self):
        free_players = self.gym.get_free_players()
        if not free_players:
            self.player_list.set('')
            self.player_list.set_values('')
            self.cur_player = ''
        else :
            free_players.sort()
            self.player_list.set_values(tuple(free_players))
            self.player_list.set('')
        
        self.set_waiting_player_list()
         
    def sel_court(self, num):
        if self.is_admin:
            return
        player = self.player_list.get().strip()
        if player not in self.player_list['value'] or player is None:
            mb.showerror('invalid player', 'Please select a valid player name in the players pull down.')
            return
        
        msg = "'{}' to join court {}?\n".format(player, num + 1)
        if not mb.askokcancel('Join court', msg):
            return
        
        if self.gym.add_to_court(player, num) :
            self.update_court(num)
            self.start_court(num)
        else:
            mb.showerror('Cannot add', 'Court {} is not open or full'.format(num + 1))
            
        # update widgets
        self.set_player_list()
        self.reset_cur_player()
    
    def update_court(self, num):
        self.courts[num].refresh()
          
    def reserve_court(self, num, period):
        court = self.gym.get_court(num)
        if court.is_playing() :
            if not mb.askyesno('reserve court', 'court {} is playing, still reserve it?'.format(num + 1)) :
                return    
        self.courts[num].reserve(period)
    
    def close_court(self, num, period):
        court = self.gym.get_court(num)
        if court.is_playing() :
            if not mb.askyesno('close court', 'court {} is playing, still close it?'.format(num + 1)) :
                return      
        self.courts[num].close(period)
        
    def open_court(self, num):
        self.courts[num].open()
        
def exit():
    if mb.askokcancel('Exit', 'Exit the application?') :
        sys.exit()

def do_nothing():
    if DEBUG :
        exit()
    pass
         
main_window = tk.Tk()
main_window.title("CBA Court Queuing System")
main_window.resizable(0, 0)
main_window.overrideredirect(True)
main_window.protocol('WM_DELETE_WINDOW', do_nothing)
main_window.attributes('-toolwindow', 1)

if not DEBUG :
    main_window.geometry("{}x{}".format(main_window.winfo_screenwidth() - 40, 
                                        main_window.winfo_screenheight() - 40))
#main_window.state('zoomed')
#main_window.geometry("800x600")

app = Application(main_window)
app.grid()

# event loop
main_window.mainloop()


# -*- coding: utf-8 -*-

# copyright 2017 Yi Wang
# don't distribute without author's explicit authorization

# MindBody API

from suds.client import Client 
import ClientRequest

class player(object):
    def __init__(self, name, pid, pin, phone = '5555555555'):
        self.name = name
        self.id = pid
        self.pin = pin
        self.phone = phone

    def get_pin(self):
        return self.pin
    
    def get_name(self):
        return self.name
    
    def get_phone(self):
        return self.phone
    
    def __str__(self):
        return("{} {} {}".format(self.name, self.id, self.pin))
    
def read_clients():
    services = ClientRequest.ClientServiceMethods()
    
    #get all clients
    result = services.GetAllClients()
    
    client_dict = Client.dict(result)
    clients = client_dict['Clients'].Client
    player_names = set()
    players = dict()
    
    num_clients = len(clients)
    for i in range(num_clients) :
        client = clients[i]
        first_name = client.FirstName.lower()
        last_name = client.LastName.lower()
        id = client.UniqueID
        
        try :
            pin = str(id)[-4:]
            name = first_name + last_name
            if len(name) > 16:
                if len(last_name) > 16:
                    name = last_name[0:15]
            else:
                temp = 16 - len(last_name)
                name = first_name[0:temp] + last_name
            
            if name in player_names:
                name += pin[-2:]
                
            player_names.add(name)
        except ValueError:
            pass
        finally:
            players[name] = player(name, id, pin)            
        
    return players



# -*- coding: utf-8 -*-

# copyright 2017 Yi Wang
# don't distribute without author's explicit authorization

# MindBody API

from suds.client import Client
import BasicRequestHelper
from datetime import datetime


class ClientServiceCalls:

    """This class contains examples of consumer methods for each ClassService method."""

    """AddArrival Methods"""

    def AddArrival(self, clientId, locationId):
        result = ClientServiceMethods().AddArrival(clientId, locationId)
        print (str(result.Message))

    """AddClientFormulaNote Methods"""

    def AddFormulaNoteToClientWithAppointment(self, clientId, appointmentId, note):
        result = ClientServiceMethods().AddClientFormulaNote(clientId, appointmentId, note)
        print (str(result))

    def AddFormulaNoteToClient(self, clientId, note):
        self.AddFormulaNoteToClientWithAppointment(clientId, None, note)

    """AddOrUpdateClient Methods"""

    def AddOrUpdateClients(self, updateAction="Fail", test=False, clients=None):
        result = ClientServiceMethods().AddOrUpdateClients(updateAction, test, clients)
        print (str(result))

    def AddCreditCardToClient(self, clientId, cc):
        """A consumer method for adding a credit card to a client's information.
           Note that adding a credit card in particular requires a secure connection 
           (https url) when setting up your service."""
        result = ClientServiceMethods().AddCreditCardToClient(clientId, cc)
        print (str(result))

    def AddNewCreditCardToClient(self, clientId,
                                 cardType,
                                 lastFour,
                                 cardNumber,
                                 holderName,
                                 expMonth,
                                 expYear,
                                 address,
                                 city,
                                 state,
                                 postalCode):
        """A consumer method for adding a new or default credit card
           to a client's information. Note that adding a credit card 
           in particular requires a secure connection (https url) when 
           setting up your service."""
        result = ClientServiceMethods().CreateAndAddCreditCardToClient(clientId,
                                                                       cardType,
                                                                       lastFour,
                                                                       cardNumber,
                                                                       holderName,
                                                                       expMonth,
                                                                       expYear,
                                                                       address,
                                                                       city,
                                                                       state,
                                                                       postalCode)
        print (str(result))

    """AddOrUpdateContactLogs Methods"""

    def UpdateContactLogText(self, clientId, text):
        result = ClientServiceMethods().UpdateContactLogText(clientId, text)
        print (str(result))

    """DeleteClientFormulaNote Methods"""

    def DeleteFormulaNote(self, clientId, formulaNoteId):
        result = ClientServiceMethods().DeleteFormulaNote(clientId, formulaNoteId)
        print (str(result))

    """GetActiveClientMemberships Methods"""

    def GetActiveClientMemberships(self, clientId, locationId=None):
        result = ClientServiceMethods().GetActiveClientMemberships(clientId, locationId)
        print (str(result))

    """GetClientAccountBalances Methods"""

    def GetRelativeClientAccountBalances(self, clientIds, balanceDate=None, classId=None):
        result = ClientServiceMethods().GetClientAccountBalances(clientIds, balanceDate, classId)
        print (str(result))

    def GetCurrentClientAccountBalances(self, clientIds, classId=None):
        self.GetRelativeClientAccountBalances(clientIds, classId=classId)

    """GetClientContactLogs Methods"""

    def GetClientContactLogs(self, clientId,
                             startDate=datetime.today(),
                             endDate=datetime.today(),
                             staffIds=None,
                             systemGenerated=False,
                             typeIds=None,
                             subtypeIds=None):
        result = ClientServiceMethods().GetClientContactLogs(clientId,
                                                             startDate,
                                                             endDate,
                                                             staffIds,
                                                             systemGenerated,
                                                             typeIds,
                                                             subtypeIds)
        print (str(result))

    """GetClientContracts Methods"""

    def GetClientContracts(self, clientId=None):
        result = ClientServiceMethods().GetClientContracts(clientId)
        print (str(result))

    """GetClientFormulaNotes Methods"""

    def GetClientFormulaNotes(self, clientId, appointmentId=None):
        result = ClientServiceMethods().GetClientFormulaNotes(clientId, appointmentId)
        print (str(result))

    """GetClientIndexes Methods"""

    def GetClientIndexes(self):
        result = ClientServiceMethods().GetClientIndexes()
        print (str(result))

    """GetClientPurchases Methods"""

    def GetClientPurchases(self, clientId, startDate=datetime.today(), endDate=datetime.today()):
        result = ClientServiceMethods().GetClientPurchases(clientId, startDate, endDate)
        print (str(result))

    """GetClientReferralTypes Methods"""

    def GetClientReferralTypes(self):
        result = ClientServiceMethods().GetClientReferralTypes()
        print (str(result))

    """GetClients Methods"""

    def GetClientsBySingleId(self, id):
        result = ClientServiceMethods().GetClientsBySingleId(id)
        print (str(result))

    def GetClientsByMultipleIds(self, ids):
        result = ClientServiceMethods().GetClientsByMultipleIds(ids)
        print (str(result))

    def GetAllClients(self):
        result = ClientServiceMethods().GetAllClients()
        return result

    def GetClientsByString(self, searchStr):
        result = ClientServiceMethods().GetClientsByString(searchStr)
        print (str(result))

    """GetClientSchedule Methods"""

    def GetClientSchedule(self, clientId, startDate=datetime.today(), endDate=datetime.today()):
        result = ClientServiceMethods().GetClientSchedule(clientId, startDate, endDate)
        print (str(result))

    """GetClientServices Methods"""

    def GetClientServices(self, clientId,
                          classId=0,
                          programIds=None,
                          sessionTypeIds=None,
                          locationIds=None,
                          visitCount=0,
                          startDate=None,
                          endDate=None,
                          showActiveOnly=False):
        result = ClientServiceMethods().GetClientServices(clientId,
                                                          classId,
                                                          programIds,
                                                          sessionTypeIds,
                                                          locationIds,
                                                          visitCount,
                                                          startDate,
                                                          endDate,
                                                          showActiveOnly)
        print (str(result))

    def GetClientServicesForPastYear(self, clientId,
                                     classId=0,
                                     programIds=None,
                                     sessionTypeIds=None,
                                     locationIds=None,
                                     visitCount=0,
                                     showActiveOnly=False):
        self.GetClientServices(clientId,
                               classId,
                               programIds,
                               sessionTypeIds,
                               locationIds,
                               visitCount,
                               BasicRequestHelper.oneYearAgo,
                               datetime.today(),
                               showActiveOnly)

    """GetClientVisits Methods"""

    def GetClientVisits(self, clientId,
                        startDate=datetime.today(),
                        endDate=datetime.today(),
                        unpaidsOnly=False):
        result = ClientServiceMethods().GetClientVisits(clientId,
                                                        startDate,
                                                        endDate,
                                                        unpaidsOnly)
        print (str(result))

    """GetContactLogTypes Methods"""

    def GetContactLogTypes(self):
        result = ClientServiceMethods().GetContactLogTypes()
        print (str(result))

    """GetCustomClientFields Methods"""

    def GetCustomClientFields(self):
        """A consumer method for retrieving all custom client fields. This contains
           an example of parsing a result to print fields out in an easier-to-read
           format than the returned XML."""
        result = ClientServiceMethods().GetCustomClientFields()

        print ("ID - Name")
        print ("------------------------------")
        for field in result.CustomClientFields.CustomClientField:
            print ("%2d - %s" % (field.ID, field.Name))

    """GetRequiredClientFields Methods"""

    def GetRequiredClientFields(self):
        result = ClientServiceMethods().GetRequiredClientFields()
        print (str(result))

    """SendUserNewPassword methods"""

    def SendUserNewPassword(self, userEmail, userFirstName, userLastName):
        result = ClientServiceMethods().SendUserNewPassword(userEmail, userFirstName, userLastName)
        if hasattr(result, "Message"):
            print (result.Message)
        else:
            print ("No error message returned. Success!")

    """UpdateClientServices Methods"""

    def UpdateClientServices(self, clientServices, test=False):
        result = ClientServiceMethods().UpdateClientServices(clientServices, test)
        print (str(result))

    """UploadClientDocument Methods"""

    def UploadClientDocument(self, clientId, fileName, fileSize):
        result = ClientServiceMethods().UploadClientDocument(clientId, fileName, fileSize)
        print (str(result))

    """ValidateLogin Methods"""

    def ValidateLogin(self, username, password):
        result = ClientServiceMethods().ValidateLogin(username, password)
        print (str(result))


class ClientServiceMethods:

    """This class contains producer methods for all ClientService methods."""
    wsdl = BasicRequestHelper.BuildWsdlUrl("Client")
    """We manually set the SoapAction field here by specifying location. Specifically,
       we need an https connection (for things such as modifying billing information
       and by default, this gets set to http."""
    service = Client(wsdl, location="https://api.mindbodyonline.com/0_5/ClientService.asmx")

    def CreateBasicRequest(self, requestName):
        return BasicRequestHelper.CreateBasicRequest(self.service, requestName)

    """AddArrival methods"""

    def AddArrival(self, clientId, locationId):
        request = self.CreateBasicRequest("AddArrivalRequest")

        request.ClientID = clientId
        request.LocationID = locationId

        return self.service.service.AddArrival(request)

    """AddClientFormulaNote methods"""

    def AddClientFormulaNote(self, clientId, appointmentId, note):
        request = self.CreateBasicRequest("AddClientFormulaNote")

        request.ClientID = clientId
        request.AppointmentID = appointmentId
        request.Note = note

        return self.service.service.AddClientFormulaNote(request)

    """AddOrUpdateClient methods"""

    def AddOrUpdateClients(self, updateAction, test, clients):
        request = self.CreateBasicRequest("AddOrUpdateClients")

        request.UpdateAction = updateAction
        request.Test = test
        request.Clients = clients

        return self.service.service.AddOrUpdateClients(request)

    def AddCreditCardToClient(self, clientId, cc):

        # Call GetClientBySingleID, then grab the first client off the list.
        clientToEdit = self.GetClientsBySingleId(clientId).Clients.Client[0]
        clientToEdit.ClientCreditCard = cc
        Clients = BasicRequestHelper.FillArrayType(self.service, [clientToEdit], "Client", "Client")

        return self.AddOrUpdateClients("Fail", False, Clients)

    def CreateAndAddCreditCardToClient(self, clientId,
                                       cardType,
                                       lastFour,
                                       number,
                                       holderName,
                                       expMonth,
                                       expYear,
                                       address,
                                       city,
                                       state,
                                       postalCode):
        """This method shows how to create a credit card item from given simple types."""
        cc = self.service.factory.create("ClientCreditCard")
        cc.CardType = cardType
        cc.LastFour = lastFour
        cc.CardNumber = number
        cc.CardHolder = holderName
        cc.ExpMonth = expMonth
        cc.ExpYear = expYear
        cc.Address = address
        cc.City = city
        cc.State = state
        cc.PostalCode = postalCode

        return self.AddCreditCardToClient(clientId, cc)

    """AddOrUpdateContactLogs methods"""

    def UpdateContactLogText(self, clientId, text):
        """This method will change the text of every contact log 
           applied to clientId in the past year to text."""
        request = self.CreateBasicRequest("AddOrUpdateContactLogs")

        contactLogs = self.GetClientContactLogsByClient(clientId, True)
        for log in contactLogs.ContactLogs.ContactLog:
            log.Text = text

        request.UpdateAction = "Fail"
        request.Test = False
        request.ContactLogs = contactLogs

        return self.service.service.AddOrUpdateContactLogs(request)

    """DeleteClientFormulaNote methods"""

    def DeleteFormulaNote(self, clientId, formulaNoteId):
        request = self.CreateBasicRequest("DeleteClientFormulaNote")

        request.ClientID = clientId
        request.FormulaNoteID = formulaNoteId

        return self.service.service.DeleteClientFormulaNote(request)

    """GetActiveClientMemberships Methods"""

    def GetActiveClientMemberships(self, clientId, locationId):
        request = self.CreateBasicRequest("GetActiveClientMemberships")

        request.ClientID = clientId
        request.LocationID = locationId

        return self.service.service.GetActiveClientMemberships(request)

    """GetClientAccountBalances methods"""

    def GetClientAccountBalances(self, clientIds, balanceDate, classId):
        request = self.CreateBasicRequest("GetClientAccountBalances")

        ClientIDs = self.service.factory.create("ArrayOfString")
        ClientIDs.string = clientIds

        request.ClientIDs = ClientIDs
        request.BalanceDate = balanceDate
        request.ClassID = classId

        return self.service.service.GetClientAccountBalances(request)

    """GetClientContactLogs methods"""

    def GetClientContactLogs(self, clientId,
                             startDate,
                             endDate,
                             staffIds,
                             systemGenerated,
                             typeIds,
                             subtypeIds):
        request = self.CreateBasicRequest("GetClientContactLogsRequest")

        request.ClientID = clientId
        request.StartDate = startDate
        request.EndDate = endDate
        request.StaffIDs = BasicRequestHelper.FillArrayType(self.service, staffIds, "Long")
        request.ShowSystemGenerated = systemGenerated
        request.TypeIDs = typeIds
        request.SubtypeIDs = subtypeIds

        return self.service.service.GetClientContactLogs(request)

    def GetClientContactLogsByClient(self, clientId, systemGenerated):
        """Convenience method to find all contact logs for a given client within the past year."""
        return self.GetClientContactLogs(clientId,
                                         BasicRequestHelper.oneYearAgo,
                                         datetime.today(),
                                         None,
                                         systemGenerated,
                                         None,
                                         None)

    """GetClientContracts methods"""

    def GetClientContracts(self, clientId):
        request = self.CreateBasicRequest("GetClientContracts")

        request.ClientID = clientId

        return self.service.service.GetClientContracts(request)

    """GetClientFormulaNotes methods"""

    def GetClientFormulaNotes(self, clientId, appointmentId):
        request = self.CreateBasicRequest("GetClientFormulaNotes")

        request.ClientID = clientId
        request.AppointmentID = appointmentId

        return self.service.service.GetClientFormulaNotes(request)

    """GetClientIndexes methods"""

    def GetClientIndexes(self):
        request = self.CreateBasicRequest("GetClientIndexesRequest")

        return self.service.service.GetClientIndexes(request)

    """GetClientPurchases methods"""

    def GetClientPurchases(self, clientId, startDate, endDate):
        request = self.CreateBasicRequest("GetClientPurchases")

        request.ClientID = clientId
        request.StartDate = startDate
        request.EndDate = endDate

        return self.service.service.GetClientPurchases(request)

    """GetClientReferralTypes methods"""

    def GetClientReferralTypes(self):
        request = self.CreateBasicRequest("GetClientReferralTypes")

        return self.service.service.GetClientReferralTypes(request)

    """GetClientVisits methods"""

    def GetClientVisits(self, clientId, startDate, endDate, unpaidsOnly):
        request = self.CreateBasicRequest("GetClientVisits")

        request.ClientID = clientId
        request.StartDate = startDate
        request.EndDate = endDate
        request.UnpaidsOnly = unpaidsOnly

        return self.service.service.GetClientVisits(request)

    """GetClients methods"""

    def GetAllClients(self):
        return self.GetClientsByString(" ")

    def GetClientsBySingleId(self, id):
        return self.GetClientsByMultipleIds([id])

    def GetClientsByString(self, searchStr):
        """Convenience method to find clients containing searchStr in their name or e-mail."""
        request = self.CreateBasicRequest("GetClientsRequest")

        # Since SearchText is just a string, we can assign it directly.
        request.SearchText = searchStr
        print(request)

        return self.service.service.GetClients(request)

    def GetClientsByMultipleIds(self, ids):
        request = self.CreateBasicRequest("GetClientsRequest")

        """Here, we create an instance of ArrayOfString (the type of ClientIDs in
           the request) and fill it with our ids before assigning it to our request."""
        clientIDs = self.service.factory.create("ArrayOfString")
        clientIDs.string = ids
        request.ClientIDs = clientIDs

        return self.service.service.GetClients(request)

    """GetClientSchedule methods"""

    def GetClientSchedule(self, clientId, startDate, endDate):
        request = self.CreateBasicRequest("GetClientSchedule")

        request.ClientID = clientId
        request.StartDate = startDate
        request.EndDate = endDate

        return self.service.service.GetClientSchedule(request)

    """GetClientServices methods"""

    def GetClientServices(self, clientId,
                          classId,
                          programIds,
                          sessionTypeIds,
                          locationIds,
                          visitCount,
                          startDate,
                          endDate,
                          showActiveOnly):
        """A few notes about GetClientServices:
            1. If you don't want to pass a Class ID in, pass 0. This acts as you would expect None to.
            2. ProgramIDs is a required field."""
        request = self.CreateBasicRequest("GetClientServices")

        request.ClientID = clientId
        request.ClassID = classId
        request.ProgramIDs = BasicRequestHelper.FillArrayType(self.service, programIds, "Int")
        request.SessionTypeIDs = BasicRequestHelper.FillArrayType(
            self.service, sessionTypeIds, "Int")
        request.LocationIDs = BasicRequestHelper.FillArrayType(self.service, locationIds, "Int")
        request.VisitCount = visitCount
        request.StartDate = startDate
        request.EndDate = endDate
        request.ShowActiveOnly = showActiveOnly

        return self.service.service.GetClientServices(request)

    """GetContactLogTypes methods"""

    def GetContactLogTypes(self):
        request = self.CreateBasicRequest("GetContactLogTypes")

        return self.service.service.GetContactLogTypes(request)

    """GetCustomClientFields methods"""

    def GetCustomClientFields(self):
        request = self.CreateBasicRequest("GetCustomClientFieldsRequest")

        return self.service.service.GetCustomClientFields(request)

    """GetRequiredClientFields methods"""

    def GetRequiredClientFields(self):
        request = self.CreateBasicRequest("GetRequiredClientFields")

        return self.service.service.GetRequiredClientFields(request)

    """SendUserNewPassword methods"""

    def SendUserNewPassword(self, userEmail, userFirstName, userLastName):
        request = self.CreateBasicRequest("SendUserNewPassword")

        request.UserEmail = userEmail
        request.UserFirstName = userFirstName
        request.UserLastName = userLastName

        return self.service.service.SendUserNewPassword(request)

    """UpdateClientServices methods"""

    def UpdateClientServices(self, clientServices, test):
        request = self.CreateBasicRequest("UpdateClientServices")

        ServiceList = self.service.factory.create("ArrayOfClientService")
        ServiceList.ClientService = clientServices

        request.ClientServices = ServiceList
        request.Test = test

        return self.service.service.UpdateClientServices(request)

    """UploadClientDocument methods"""

    def UploadClientDocument(self, clientId, fileName, fileSize):
        request = self.CreateBasicRequest("UploadClientDocument")

        request.ClientID = clientId
        request.FileName = fileName
        request.Bytes = fileSize

        return self.service.service.UploadClientDocument(request)

    """ValidateLogin methods"""

    def ValidateLogin(self, username, password):
        request = self.CreateBasicRequest("ValidateLogin")

        request.Username = username
        request.Password = password

        return self.service.service.ValidateLogin(request)    



"""
Created by Mitja Martini on 2008-11-29.
Updated by Russell Adams, 2011/01/24 to support Python 3 and Combobox.
Updated by Dominic Kexel to use Tkinter and ttk instead of tkinter and tkinter.ttk
   Licensed same as original (not specified?), or public domain, whichever is less restrictive.
"""
import sys
import os
import tkinter as tk
import tkinter.ttk as ttk

__version__ = "1.1"

class smart_combobox(ttk.Combobox):

    def set_values(self, values):
        """Use our completion list as our drop down selection menu, arrows move through menu."""
        self._values = sorted(values, key=str.lower) # Work with a sorted list
        self._hits = []
        self._hit_index = 0
        self.position = 0
        self.bind('<KeyRelease>', self.handle_keyrelease)
        self['values'] = self._values  # Setup our popup menu

    def autocomplete(self, delta=0):
        """autocomplete the Combobox, delta may be 0/1/-1 to cycle through possible hits"""
        if delta: # need to delete selection otherwise we would fix the current position
            self.delete(self.position, tk.END)
        else: # set position to end so selection starts where textentry ended
            self.position = len(self.get())
        # collect hits
        _hits = []
        for element in self._values:
            if element.lower().startswith(self.get().lower()): # Match case insensitively
                _hits.append(element)
        # if we have a new hit list, keep this in mind
        if _hits != self._hits:
            self._hit_index = 0
            self._hits=_hits
        # only allow cycling if we are in a known hit list
        if _hits == self._hits and self._hits:
            self._hit_index = (self._hit_index + delta) % len(self._hits)
        # now finally perform the auto completion
        if self._hits:
            self.delete(0,tk.END)
            self.insert(0,self._hits[self._hit_index])
            self.select_range(self.position,tk.END)

    def handle_keyrelease(self, event):
        """event handler for the keyrelease event on this widget"""
        if event.keysym == "BackSpace":
            self.delete(self.index(tk.INSERT), tk.END)
            self.position = self.index(tk.END)
        if event.keysym == "Left":
            if self.position < self.index(tk.END): # delete the selection
                self.delete(self.position, tk.END)
            else:
                self.position = self.position-1 # delete one character
                self.delete(self.position, tk.END)
        if event.keysym == "Right":
            self.position = self.index(tk.END) # go to end (no selection)
        if len(event.keysym) == 1:
            self.autocomplete()
        # No need for up/down, we'll jump to the popup
        # list at the position of the autocompletion



# copyright 2017 Yi Wang
# don't distribute without author's explicit authorization

from distutils.core import setup

setup(
    name='badminton_queuing_system',
    packages=['badminton_queuing_system'],
    version='0.12',
    description='Badminton Queuing system tailored for CBA',
    url='http://github.com/yiw2000/badminton_queuing_system',
    author='Yi Wang',
    author_email='yiw2000@gmail.com',
)